comment /******************************************************************\
*                            FYS OS version 2.0                            *
* FILE: onefat.asm                                                         *
* FILE Ver: 1.00                                                           *
*                                                                          *
* This code is freeware, not public domain.  Please use respectfully.      *
*                                                                          *
* Changes a 1.44m DOS formatted floppy to use only one FAT.                *
*                                                                          *
*  Built with:  NBASM ver 00.26.31                                         *
*                 http:\\www.frontiernet.net\~fys\newbasic.htm             *
* Last Update:  15 Jan 2010                                                *
*                                                                          *
****************************************************************************
* Notes:                                                                   *
* -This utility will modify the boot sector's bpb.fats entry to 1 instead  *
*  of 2.  It will then 'erase' the second fat therefore placing the root   *
*  directory 9 sectors closer to the front of the disk.                    *
*                                                                          *
* -This only works on an already formatted 1.44m DOS FAT12 disk.           *
*                                                                          *
* -This will erase any contents already there, so make sure the disk is    *
*  freshly formatted.                                                      *
*                                                                          *
*                                                                          *
*                                                                          *
****************************************************************************
* Assumptions:                                                             *
* -You have a i186 compatible machine.                                     *
* -This util uses drive a:, ie.  BIOS drive dl = 00h                       *
*   Change the EQUate below for b: (dl = 1)                                *
* -This utility assumes that reading and writing to the floppy will not    *
*  be interrupted or interpretted as a virus, and will be allowed to read  *
*  and write at will.                                                      *
*                                                                          *
*                                                                          *
*                                                                          *
*                                                                          *
****************************************************************************
*                                                                          *
* TODO:                                                                    *
*  -                                                                       *
*                                                                          *
***************************************************************************/

DRIVE       equ  0   ; drive a: (dl = 0)
SECT_TRACK  equ 18   ; sectors per track
HEADS       equ  2   ; heads

READ   equ  (02<<8)
WRITE  equ  (03<<8)

.model tiny
.186
.code
.rmode

; =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-
           org 100h

           ; =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-
           ; es -> buffer
           mov  ax,ds
           mov  cx,offset buffer
           shr  cx,4
           add  ax,cx
           mov  es,ax

           ; =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-
           ; first read in the boot sector
           xor  dx,dx
           xor  ax,ax
           mov  cx,1
           mov  si,READ
           call trans_sectors
           mov  byte item_num,'1'
           jc   short disk_error

           ; =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-
           ; change the fats entry to 1
           mov  byte es:[10h],1

           ; =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-
           ; write it back
           mov  si,WRITE
           call trans_sectors
           mov  byte item_num,'2'
           jc   short disk_error

; =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-

           ; =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-
           ; now clear out the buffer
           ; remember that es:0000h -> buffer
           xor  di,di
           mov  cx,(256*9)
           xor  ax,ax
           rep
           stosw

           ; =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-
           ; now write it to the second fat location, in turn
           ; "deleting" the second fat and moving the root directory
           ; up 9 sectors
           xor  dx,dx
           mov  ax,10
           mov  cx,9
           mov  si,WRITE
           call trans_sectors
           mov  byte item_num,'3'
           jc   short disk_error

; =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-

           ; =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-
           ; now the disk should work fine under any FAT12 compliant
           ; OS that doesn't assume 2 fats.

           ; =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-
           ; exit to DOS
           .exit 0

disk_error:
           mov  dx,offset diskerror_str
           mov  ah,09
           int  21h

           .exit 1


diskerror_str db 13,10,'There was an error at transfer item: '
item_num      db ' $'

; =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-
; This routine transfers CX sectors using the bios interrupt 13h service
; Since we don't know if the current bios can span heads and cylinders,
;  with multiple counts in CX, we will transfer one sector at a time,
;  manually updating DH, and CX as we go.
; On entry:
;  DX:AX = starting sector in LBA format
;     ES ->segment to transfer (es:0000h)
;     CX = count of sectors to transfer
;     SI = 0200h = READ, 0300h = WRITE
.align 4
trans_sectors proc near uses ax bx cx dx es
trans_loop:
           push dx
           push ax
           push cx
           call lba_to_chs         ; dx:ax(lba) -> (ax/cx/dx)(chs)
           mov  ax,0003h           ; three try's
chck_loop: push ax
           mov  dl,DRIVE           ; dl = drive
           xor  bx,bx              ; es:0000h
           mov  ax,si              ; si = command
           mov  al,01h             ; al = one sector at a time
           int  13h                ; do the read/write
           jnc  short int13_no_error
           push cx
           push dx
           xor  ah,ah
           int  13h                ; reset disk
           pop  dx
           pop  cx
           pop  ax
           dec  ax
           jnz  short chck_loop    ; try again

           stc
           ret

int13_no_error:
           mov  ax,es              ;
           add  ax,20h             ; point to next position
           mov  es,ax              ;
           pop  ax                 ; restore try count
           pop  cx                 ; restore sector count
           pop  ax                 ; restore current LBA
           pop  dx                 ; (dx:ax)
           add  ax,1               ; inc to next LBA sector
           adc  dx,0               ;
           dec  cx                 ; decrement counter
           jnz  short trans_loop   ; read another one

           clc
           ret
trans_sectors endp

; =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-
; This routine converts LBA (DX:AX) to CHS
; Sector   = (LBA mod SPT)+1
; Head     = (LBA  /  SPT) mod Heads
; Cylinder = (LBA  /  SPT)  /  Heads
;    (SPT = Sectors per Track)
lba_to_chs proc near               ; dx:ax = LBA
           mov  cx,SECT_TRACK      ; sectors per track 
           call div32_16           ; dx:ax = dx:ax / cx  with cx = remdr.
           push cx                 ; save sectors
           mov  cx,HEADS           ; heads per cylinder
           call div32_16           ; dx:ax = dx:ax / cx  with cx = remdr.
           pop  dx                 ; dx = sector
           inc  dx                 ; sectors are one (1) based
           mov  dh,cl
           mov  ch,al
           mov  cl,ah
           ror  cl,2
           and  cl,11000000b
           and  dl,00111111b
           or   cl,dl
           ret
lba_to_chs endp

; =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-
; dx:ax = dx:ax / cx  with cx = remainder
div32_16   proc near uses bx
           mov  bx,dx              ;
           xchg bx,ax              ;
           xor  dx,dx              ;
           div  cx                 ; divide high order first
           xchg bx,ax              ;
           div  cx                 ; remainder will be in dx
           mov  cx,dx              ;  save remainder
           mov  dx,bx              ; full 32-bit answer
           ret                     ;
div32_16   endp




; =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-
.para
buffer       dup (512*9),?

.end
